@@ -1,5 +1,5 @@
 package POEx::IRC::Backend;
-$POEx::IRC::Backend::VERSION = '0.029001';
+$POEx::IRC::Backend::VERSION = '0.030001';
 use strictures 2;
 
 use Carp;
@@ -139,11 +139,8 @@ has ssl_context => (
 );
 
 
-
 sub spawn {
   my ($class, %args) = @_;
-  # FIXME probably ssl_opts should be an attr and settable from here for
-  # backwards-compat:
   my $ssl_opts = delete $args{ssl_opts};
   my $self = blessed $class ? $class : $class->new(%args);
 
@@ -201,8 +198,7 @@ sub _start {
   $kernel->refcount_increment( $self->session_id, "IRCD Running" );
 }
 
-sub _stop {
-}
+sub _stop {}
 
 sub shutdown {
   my $self = shift;
@@ -234,6 +230,123 @@ sub _register_controller {
   $kernel->post( $self->controller => ircsock_registered => $self );
 }
 
+sub _idle_alarm {
+  my ($kernel, $self, $w_id) = @_[KERNEL, OBJECT, ARG0];
+  my $this_conn = $self->wheels->{$w_id} || return;
+
+  $kernel->post( $self->controller => ircsock_connection_idle => $this_conn );
+
+  $this_conn->alarm_id(
+    $kernel->delay_set( _idle_alarm => $this_conn->idle, $w_id )
+  );
+}
+
+sub create_listener {
+  my $self = shift;
+  $poe_kernel->post( $self->session_id => create_listener => @_ );
+  $self
+}
+
+sub _create_listener {
+  my ($kernel, $self, %args) = @_[KERNEL, OBJECT, ARG0 .. $#_];
+  $args{lc $_} = delete $args{$_} for keys %args;
+
+  my $bindaddr  = delete $args{bindaddr} || '0.0.0.0';
+  my $bindport  = delete $args{port}     || 0;
+
+  my $protocol = ( delete $args{ipv6} || ip_is_ipv6($bindaddr) ) ? 6 : 4;
+
+  my $wheel = POE::Wheel::SocketFactory->new(
+    SocketDomain => ($protocol == 6 ? AF_INET6 : AF_INET),
+    BindAddress  => $bindaddr,
+    BindPort     => $bindport,
+    SuccessEvent => 
+      ( $protocol == 6 ? '_accept_conn_v6' : '_accept_conn_v4' ),
+    FailureEvent => '_accept_fail',
+    Reuse        => 1,
+  );
+
+  my $id = $wheel->ID;
+
+  my $listener = POEx::IRC::Backend::Listener->new(
+    protocol => $protocol,
+    wheel => $wheel,
+    addr  => $bindaddr,
+    port  => $bindport,
+    idle  => ( delete($args{idle}) || 180 ),
+    ssl   => ( delete($args{ssl})  || 0 ),
+    ( keys %args ? (args => \%args) : () ),
+  );
+
+  $self->listeners->{$id} = $listener;
+
+  ## Real bound port/addr
+  my (undef, $port) = get_unpacked_addr( $wheel->getsockname );
+  $listener->set_port($port) if $port;
+
+  $kernel->post( $self->controller => ircsock_listener_created => $listener )
+}
+
+sub remove_listener {
+  my $self = shift;
+  $poe_kernel->post( $self->session_id => remove_listener => @_ );
+  $self
+}
+
+sub _remove_listener {
+  my ($kernel, $self) = @_[KERNEL, OBJECT];
+  my %args = @_[ARG0 .. $#_];
+  $args{lc $_} = delete $args{$_} for keys %args;
+
+  if (defined $args{listener} && $self->listeners->{ $args{listener} }) {
+    my $listener = delete $self->listeners->{ $args{listener} };
+    $listener->clear_wheel;
+    $kernel->post( $self->controller =>
+      ircsock_listener_removed => $listener
+    );
+    return
+  }
+
+  my @removed;
+
+  LISTENER: for my $id (keys %{ $self->listeners }) {
+    my $listener = $self->listeners->{$id};
+    if (defined $args{port} && defined $args{addr}) {
+      if ($args{addr} eq $listener->addr && $args{port} eq $listener->port) {
+        delete $self->listeners->{$id};
+        push @removed, $listener;
+        next LISTENER
+      }
+    } elsif (defined $args{addr} && $args{addr} eq $listener->addr) {
+      delete $self->listeners->{$id};
+      push @removed, $listener;
+    } elsif (defined $args{port} && $args{port} eq $listener->port) {
+      delete $self->listeners->{$id};
+      push @removed, $listener;
+    }
+  }
+
+  for my $listener (@removed) {
+    $listener->clear_wheel;
+    $kernel->post( $self->controller => 
+      ircsock_listener_removed => $listener 
+    );
+  }
+}
+
+sub _accept_fail {
+  my ($kernel, $self) = @_[KERNEL, OBJECT];
+  my ($op, $errnum, $errstr, $listener_id) = @_[ARG0 .. ARG3];
+
+  my $listener = delete $self->listeners->{$listener_id};
+  if ($listener) {
+    $listener->clear_wheel;
+    $kernel->post( $self->controller => 
+      ircsock_listener_failure => $listener, $op, $errnum, $errstr
+    );
+  }
+}
+
 sub _accept_conn {
   ## Accepted connection to a listener.
   my ($self, $sock, $p_addr, $p_port, $listener_id) = @_[OBJECT, ARG0 .. ARG3];
@@ -300,141 +413,9 @@ sub _accept_conn {
   );
 }
 
-sub _idle_alarm {
-  my ($kernel, $self, $w_id) = @_[KERNEL, OBJECT, ARG0];
-  my $this_conn = $self->wheels->{$w_id} || return;
-
-  $kernel->post( $self->controller => ircsock_connection_idle => $this_conn );
-
-  $this_conn->alarm_id(
-    $kernel->delay_set( _idle_alarm => $this_conn->idle, $w_id )
-  );
-}
-
-sub _accept_fail {
-  my ($kernel, $self) = @_[KERNEL, OBJECT];
-  my ($op, $errnum, $errstr, $listener_id) = @_[ARG0 .. ARG3];
-
-  my $listener = delete $self->listeners->{$listener_id};
-
-  if ($listener) {
-    $listener->clear_wheel;
-
-    $kernel->post( $self->controller => 
-      ircsock_listener_failure => $listener, $op, $errnum, $errstr
-    );
-  }
-}
-
-
-sub create_listener {
-  my $self = shift;
-
-  $poe_kernel->post( $self->session_id => create_listener => @_ );
-
-  $self
-}
-
-sub _create_listener {
-  my ($kernel, $self, %args) = @_[KERNEL, OBJECT, ARG0 .. $#_];
-  $args{lc $_} = delete $args{$_} for keys %args;
-
-  my $bindaddr  = delete $args{bindaddr} || '0.0.0.0';
-  my $bindport  = delete $args{port}     || 0;
-
-  my $protocol = ( delete $args{ipv6} || ip_is_ipv6($bindaddr) ) ? 6 : 4;
-
-  my $wheel = POE::Wheel::SocketFactory->new(
-    SocketDomain => ($protocol == 6 ? AF_INET6 : AF_INET),
-    BindAddress  => $bindaddr,
-    BindPort     => $bindport,
-    SuccessEvent => 
-      ( $protocol == 6 ? '_accept_conn_v6' : '_accept_conn_v4' ),
-    FailureEvent => '_accept_fail',
-    Reuse        => 1,
-  );
-
-  my $id = $wheel->ID;
-
-  my $listener = POEx::IRC::Backend::Listener->new(
-    protocol => $protocol,
-    wheel => $wheel,
-    addr  => $bindaddr,
-    port  => $bindport,
-    idle  => ( delete($args{idle}) || 180 ),
-    ssl   => ( delete($args{ssl})  || 0 ),
-    ( keys %args ? (args => \%args) : () ),
-  );
-
-  $self->listeners->{$id} = $listener;
-
-  ## Real bound port/addr
-  my (undef, $port) = get_unpacked_addr( $wheel->getsockname );
-  $listener->set_port($port) if $port;
-
-  $kernel->post( $self->controller => 
-    ircsock_listener_created => $listener
-  );
-}
-
-sub remove_listener {
-  my $self = shift;
-
-  $poe_kernel->post( $self->session_id => remove_listener => @_ );
-
-  $self
-}
-
-sub _remove_listener {
-  my ($kernel, $self) = @_[KERNEL, OBJECT];
-  my %args = @_[ARG0 .. $#_];
-
-  $args{lc $_} = delete $args{$_} for keys %args;
-
-  if (defined $args{listener} && $self->listeners->{ $args{listener} }) {
-    my $listener = delete $self->listeners->{ $args{listener} };
-
-    $listener->clear_wheel;
-
-    $kernel->post( $self->controller =>
-      ircsock_listener_removed => $listener
-    );
-
-    return
-  }
-
-  my @removed;
-
-  LISTENER: for my $id (keys %{ $self->listeners }) {
-    my $listener = $self->listeners->{$id};
-    if (defined $args{port} && defined $args{addr}) {
-      if ($args{addr} eq $listener->addr && $args{port} eq $listener->port) {
-        delete $self->listeners->{$id};
-        push @removed, $listener;
-        next LISTENER
-      }
-    } elsif (defined $args{addr} && $args{addr} eq $listener->addr) {
-      delete $self->listeners->{$id};
-      push @removed, $listener;
-    } elsif (defined $args{port} && $args{port} eq $listener->port) {
-      delete $self->listeners->{$id};
-      push @removed, $listener;
-    }
-  }
-
-  for my $listener (@removed) {
-    $listener->clear_wheel;
-    $kernel->post( $self->controller => 
-      ircsock_listener_removed => $listener 
-    );
-  }
-}
-
 sub create_connector {
   my $self = shift;
-
   $poe_kernel->post( $self->session_id => create_connector => @_ );
-
   $self
 }
 
@@ -455,7 +436,7 @@ sub _create_connector {
   my $remote_addr = delete $args{remoteaddr};
   my $remote_port = delete $args{remoteport};
 
-  die "create_connector expects a RemoteAddr and RemotePort\n"
+  die "create_connector expects a remoteaddr and remoteport\n"
     unless defined $remote_addr and defined $remote_port;
 
   my $protocol =
@@ -525,7 +506,7 @@ sub _connector_up {
     try {
       die "Failed to load POE::Component::SSLify" unless $self->has_ssl_support;
       $sock = POE::Component::SSLify::Client_SSLify(
-        $sock, 0, 0, $self->ssl_context
+        $sock, undef, undef, $self->ssl_context
       );
       $using_ssl = 1
     } catch {
@@ -595,7 +576,6 @@ sub _ircsock_input {
   $poe_kernel->delay_adjust( $this_conn->alarm_id, $this_conn->idle )
     if $this_conn->has_alarm_id;
 
-  ## FIXME configurable raw events?
   $poe_kernel->post( $_[OBJECT]->controller => 
     ircsock_input => $this_conn, IRC::Message::Object->new(%{ $_[ARG0] })
   );
@@ -665,11 +645,10 @@ sub send {
 }
 
 sub disconnect {
-  ## Mark a wheel for disconnection.
+  # Mark a wheel for disconnection at next flush.
   my ($self, $w_id, $str) = @_;
   $w_id = $w_id->wheel_id if blessed $w_id;
-
-  confess "disconnect() needs an (extant) wheel ID or ->wheel_id"
+  confess "disconnect() needs an (extant) wheel ID or Connect object"
     unless defined $w_id;
 
   # Application code should probably check $conn->has_wheel before trying to
@@ -682,10 +661,24 @@ sub disconnect {
     return
   }
 
-  $self->wheels->{$w_id}->is_disconnecting(
-    $str || "Client disconnect"
-  );
+  $self->wheels->{$w_id}->is_disconnecting($str // "Client disconnect");
+  $self
+}
 
+sub disconnect_now {
+  my ($self, $w_id, $str) = @_;
+  $w_id = $w_id->wheel_id if blessed $w_id;
+  confess "disconnect_now needs an (extant) wheel ID or Connect object"
+    unless defined $w_id;
+
+  unless (defined $self->wheels->{$w_id}) {
+    carp "Attempting to disconnect() unknown wheel '$w_id'\n",
+      " This warning may be spurious. Your wheel may have died of natural causes.\n",
+      " Try checking '\$conn->has_wheel' before calling disconnect()." ;
+    return
+  }
+
+  $self->_disconnected($w_id, $str // "Client disconnect");
   $self
 }
 
@@ -1010,11 +1003,16 @@ L<POEx::IRC::Backend::Connect> still has an active wheel attached:
     $backend->disconnect( $this_conn )
   }
 
-Note that disconnection happens after a buffer flush; if your software does
-not perform entirely like a traditional platform (server implementations
-will typically send C<< ERROR: Closing Link >> or similar to clients marked
-for disconnection, which will trigger a buffer flush) you may currently experience
-"late" disconnects. This behavior may be subject to change in future versions.
+Note that disconnection typically happens after a buffer flush; if your
+software does not perform entirely like a traditional platform (server
+implementations will typically send C<< ERROR: Closing Link >> or similar to
+clients marked for disconnection, which will trigger a buffer flush) you may
+currently experience "late" disconnects. See L</disconnect_now>.
+
+=head3 disconnect_now
+
+Like L</disconnect>, but attempt to destroy the wheel immediately (without
+waiting for a buffer flush).
 
 =head3 send
 
@@ -1221,12 +1219,10 @@ C<$_[ARG0]> is the Backend's C<$self> object.
 
 =head1 BUGS
 
-Probably lots. Please report them via RT, e-mail, or GitHub
+Probably lots. Please report them via RT, e-mail, IRC
+(C<irc.cobaltirc.org#perl>), or GitHub
 (L<http://github.com/avenj/poex-irc-backend>).
 
-Tests are a bit incomplete, as of this writing. 
-Zlib and SSL are mostly untested.
-
 =head1 SEE ALSO
 
 L<POEx::IRC::Backend::Connect>
